(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.fetchInject = factory());
}(this, (function () { 'use strict';

  /**
   * Copyright (C) 2017—2018 Josh Habdas <jhabdas@protonmail.com>
   *
   * This work is free. You can redistribute it and/or modify it under the
   * terms of the Do What The Fuck You Want To Public License, Version 2,
   * as published by Sam Hocevar. See the COPYING file for more details.
   */

  const head = (function(i,s,o,g,r,a,m){a=s.createElement(o),m=s.getElementsByTagName(o)[0];a.appendChild(s.createTextNode(g.text));a.onload=r(g);m?m.parentNode.insertBefore(a,m):s.head.appendChild(a);}); // eslint-disable-line

  /**
   * Fetch Inject module.
   *
   * @module fetchInject
   * @license WTFPL
   * @param {(USVString[]|Request[])} inputs Resources you wish to fetch.
   * @param {Promise} [promise] A promise to await before attempting injection.
   * @throws {Promise<TypeError>} Rejects with error on invalid arguments.
   * @throws {Promise<Error>} Whatever `fetch` decides to throw.
   * @throws {SyntaxError} Via DOM upon attempting to parse unexpected tokens.
   * @returns {Promise<Object[]>} A promise which resolves to an `Array` of
   *     Objects containing `Response` `Body` properties used by the module.
   */
  const fetchInject = function (inputs, promise) {
    if (!arguments.length) throw new TypeError("Failed to execute 'fetchInject': 1 argument required but only 0 present.")
    ;[...arguments].forEach((arg, idx) => {
      if (idx === 0 && arg.constructor !== Array) throw new TypeError("Failed to execute 'fetchInject': argument 1 must be of type 'Array'.")
      if (idx === 1 && arg.constructor !== Promise) throw new TypeError("Failed to execute 'fetchInject': argument 2 must be of type 'Promise'.")
    });

    const resources = [];
    const deferreds = promise ? [].concat(promise) : [];
    const thenables = [];

    inputs.forEach(input => deferreds.push(
      window.fetch(input).then(res => {
        return [res.clone().text(), res.blob()]
      }).then(promises => {
        return Promise.all(promises).then(resolved => {
          resources.push({ text: resolved[0], blob: resolved[1] });
        })
      })
    ));

    return Promise.all(deferreds).then(() => {
      resources.forEach(resource => {
        thenables.push({ then: resolve => {
          resource.blob.type.includes('text/css')
            ? head(window, document, 'style', resource, resolve)
            : head(window, document, 'script', resource, resolve);
        }});
      });
      return Promise.all(thenables)
    })
  };

  return fetchInject;

})));
